@@ -5,10 +5,9 @@
 #include <oqs/sig_sphincs.h>
 
 #if defined(OQS_ENABLE_SIG_sphincs_shake_256f_simple)
-
 OQS_SIG *OQS_SIG_sphincs_shake_256f_simple_new(void) {
 
-	OQS_SIG *sig = malloc(sizeof(OQS_SIG));
+	OQS_SIG *sig = OQS_MEM_malloc(sizeof(OQS_SIG));
 	if (sig == NULL) {
 		return NULL;
 	}
@@ -17,6 +16,8 @@ OQS_SIG *OQS_SIG_sphincs_shake_256f_simple_new(void) {
 
 	sig->claimed_nist_level = 5;
 	sig->euf_cma = true;
+	sig->suf_cma = false;
+	sig->sig_with_ctx_support = false;
 
 	sig->length_public_key = OQS_SIG_sphincs_shake_256f_simple_length_public_key;
 	sig->length_secret_key = OQS_SIG_sphincs_shake_256f_simple_length_secret_key;
@@ -25,6 +26,8 @@ OQS_SIG *OQS_SIG_sphincs_shake_256f_simple_new(void) {
 	sig->keypair = OQS_SIG_sphincs_shake_256f_simple_keypair;
 	sig->sign = OQS_SIG_sphincs_shake_256f_simple_sign;
 	sig->verify = OQS_SIG_sphincs_shake_256f_simple_verify;
+	sig->sign_with_ctx_str = OQS_SIG_sphincs_shake_256f_simple_sign_with_ctx_str;
+	sig->verify_with_ctx_str = OQS_SIG_sphincs_shake_256f_simple_verify_with_ctx_str;
 
 	return sig;
 }
@@ -87,4 +90,19 @@ OQS_API OQS_STATUS OQS_SIG_sphincs_shake_256f_simple_verify(const uint8_t *messa
 #endif
 }
 
+OQS_API OQS_STATUS OQS_SIG_sphincs_shake_256f_simple_sign_with_ctx_str(uint8_t *signature, size_t *signature_len, const uint8_t *message, size_t message_len, const uint8_t *ctx_str, size_t ctx_str_len, const uint8_t *secret_key) {
+	if (ctx_str == NULL && ctx_str_len == 0) {
+		return OQS_SIG_sphincs_shake_256f_simple_sign(signature, signature_len, message, message_len, secret_key);
+	} else {
+		return OQS_ERROR;
+	}
+}
+
+OQS_API OQS_STATUS OQS_SIG_sphincs_shake_256f_simple_verify_with_ctx_str(const uint8_t *message, size_t message_len, const uint8_t *signature, size_t signature_len, const uint8_t *ctx_str, size_t ctx_str_len, const uint8_t *public_key) {
+	if (ctx_str == NULL && ctx_str_len == 0) {
+		return OQS_SIG_sphincs_shake_256f_simple_verify(message, message_len, signature, signature_len, public_key);
+	} else {
+		return OQS_ERROR;
+	}
+}
 #endif
