@@ -3,39 +3,114 @@
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
+#if defined(_WIN32)
 #include <string.h>
+#define strcasecmp _stricmp
+#else
+#include <strings.h>
+#endif
 
 #include <oqs/oqs.h>
-
-#if OQS_USE_PTHREADS_IN_TESTS
+#include <oqs/sha3.h>
+#if OQS_USE_PTHREADS
 #include <pthread.h>
 #endif
 
-#ifdef OQS_ENABLE_TEST_CONSTANT_TIME
-#include <valgrind/memcheck.h>
-#define OQS_TEST_CT_CLASSIFY(addr, len)  VALGRIND_MAKE_MEM_UNDEFINED(addr, len)
-#define OQS_TEST_CT_DECLASSIFY(addr, len)  VALGRIND_MAKE_MEM_DEFINED(addr, len)
-#else
-#define OQS_TEST_CT_CLASSIFY(addr, len)
-#define OQS_TEST_CT_DECLASSIFY(addr, len)
+#ifdef OQS_ENABLE_KEM_ML_KEM
+#define MLKEM_SECRET_LEN      32
 #endif
 
 #include "system_info.c"
+#include "test_helpers.h"
 
-/* Displays hexadecimal strings */
-static void OQS_print_hex_string(const char *label, const uint8_t *str, size_t len) {
-	printf("%-20s (%4zu bytes):  ", label, len);
-	for (size_t i = 0; i < (len); i++) {
-		printf("%02X", str[i]);
+#ifdef OQS_ENABLE_KEM_ML_KEM
+/* mlkem rejection key testcase */
+static bool mlkem_rej_testcase(OQS_KEM *kem, uint8_t *ciphertext, uint8_t *secret_key) {
+	// sanity checks
+	if ((kem == NULL) || (ciphertext == NULL) || (secret_key == NULL)) {
+		fprintf(stderr, "ERROR: inputs NULL!\n");
+		return false;
 	}
-	printf("\n");
+	// Only run tests for ML-KEM
+	if (!(strcasecmp(kem->method_name, OQS_KEM_alg_ml_kem_512) == 0 ||
+	        strcasecmp(kem->method_name, OQS_KEM_alg_ml_kem_768) == 0 ||
+	        strcasecmp(kem->method_name, OQS_KEM_alg_ml_kem_1024) == 0)) {
+		return true;
+	}
+	// Buffer to hold z and c. z is always 32 bytes
+	uint8_t *buff_z_c = NULL;
+	bool retval = false;
+	OQS_STATUS rc;
+	int rv;
+	size_t length_z_c = 32 + kem->length_ciphertext;
+	buff_z_c = OQS_MEM_malloc(length_z_c);
+	if (buff_z_c == NULL) {
+		fprintf(stderr, "ERROR: OQS_MEM_malloc failed\n");
+		return false;
+	}
+	// Scenario 1: Test rejection key by corrupting the secret key
+	secret_key[0] += 1;
+	uint8_t shared_secret_r[MLKEM_SECRET_LEN]; // expected output
+	uint8_t shared_secret_d[MLKEM_SECRET_LEN]; // calculated output
+	memcpy(buff_z_c, &secret_key[kem->length_secret_key - 32], 32);
+	memcpy(&buff_z_c[MLKEM_SECRET_LEN], ciphertext, kem->length_ciphertext);
+	// Calculate expected secret in case of corrupted cipher: shake256(z || c)
+	OQS_SHA3_shake256(shared_secret_r, MLKEM_SECRET_LEN, buff_z_c, length_z_c);
+	rc = OQS_KEM_decaps(kem, shared_secret_d, ciphertext, secret_key);
+	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+	if (rc != OQS_SUCCESS) {
+		fprintf(stderr, "ERROR: OQS_KEM_decaps failed for rejection testcase scenario 1\n");
+		goto cleanup;
+	}
+	OQS_TEST_CT_DECLASSIFY(shared_secret_d, MLKEM_SECRET_LEN);
+	OQS_TEST_CT_DECLASSIFY(shared_secret_r, MLKEM_SECRET_LEN);
+	rv = memcmp(shared_secret_d, shared_secret_r, MLKEM_SECRET_LEN);
+	if (rv != 0) {
+		fprintf(stderr, "ERROR: shared secrets are not equal for rejection key in decapsulation scenario 1\n");
+		OQS_print_hex_string("shared_secret_d", shared_secret_d, MLKEM_SECRET_LEN);
+		OQS_print_hex_string("shared_secret_r", shared_secret_r, MLKEM_SECRET_LEN);
+		goto cleanup;
+	}
+	secret_key[0] -= 1; // Restore private key
+	memset(buff_z_c, 0, length_z_c); // Reset buffer
+
+	// Scenario 2: Test rejection key by corrupting the ciphertext
+	ciphertext[0] += 1;
+	memcpy(buff_z_c, &secret_key[kem->length_secret_key - 32], 32);
+	memcpy(&buff_z_c[MLKEM_SECRET_LEN], ciphertext, kem->length_ciphertext);
+
+	// Calculate expected secret in case of corrupted cipher: shake256(z || c)
+	OQS_SHA3_shake256(shared_secret_r, MLKEM_SECRET_LEN, buff_z_c, length_z_c);
+	rc = OQS_KEM_decaps(kem, shared_secret_d, ciphertext, secret_key);
+	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+	if (rc != OQS_SUCCESS) {
+		fprintf(stderr, "ERROR: OQS_KEM_decaps failed for rejection testcase scenario 2\n");
+		goto cleanup;
+	}
+	OQS_TEST_CT_DECLASSIFY(shared_secret_d, MLKEM_SECRET_LEN);
+	OQS_TEST_CT_DECLASSIFY(shared_secret_r, MLKEM_SECRET_LEN);
+	rv = memcmp(shared_secret_d, shared_secret_r, MLKEM_SECRET_LEN);
+	if (rv != 0) {
+		fprintf(stderr, "ERROR: shared secrets are not equal for rejection key in decapsulation scenario 2\n");
+		OQS_print_hex_string("shared_secret_d", shared_secret_d, MLKEM_SECRET_LEN);
+		OQS_print_hex_string("shared_secret_r", shared_secret_r, MLKEM_SECRET_LEN);
+		goto cleanup;
+	}
+	ciphertext[0] -= 1; // Restore ciphertext
+	retval = true;
+cleanup:
+	if (buff_z_c) {
+		OQS_MEM_secure_free(buff_z_c, length_z_c);
+	}
+	return retval;
 }
+#endif //OQS_ENABLE_KEM_ML_KEM
 
 typedef struct magic_s {
 	uint8_t val[31];
 } magic_t;
 
-static OQS_STATUS kem_test_correctness(const char *method_name) {
+static OQS_STATUS kem_test_correctness(const char *method_name, bool derand) {
 
 	OQS_KEM *kem = NULL;
 	uint8_t *public_key = NULL;
@@ -43,6 +118,7 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 	uint8_t *ciphertext = NULL;
 	uint8_t *shared_secret_e = NULL;
 	uint8_t *shared_secret_d = NULL;
+	uint8_t *seed = NULL;
 	OQS_STATUS rc, ret = OQS_ERROR;
 	int rv;
 
@@ -57,18 +133,22 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 		goto err;
 	}
 
-	printf("================================================================================\n");
-	printf("Sample computation for KEM %s\n", kem->method_name);
-	printf("================================================================================\n");
+	if (!derand) {
+		printf("================================================================================\n");
+		printf("sample computation for KEM %s\n", kem->method_name);
+		printf("Version source: %s\n", kem->alg_version);
+		printf("================================================================================\n");
+	}
 
-	public_key = malloc(kem->length_public_key + 2 * sizeof(magic_t));
-	secret_key = malloc(kem->length_secret_key + 2 * sizeof(magic_t));
-	ciphertext = malloc(kem->length_ciphertext + 2 * sizeof(magic_t));
-	shared_secret_e = malloc(kem->length_shared_secret + 2 * sizeof(magic_t));
-	shared_secret_d = malloc(kem->length_shared_secret + 2 * sizeof(magic_t));
+	public_key = OQS_MEM_malloc(kem->length_public_key + 2 * sizeof(magic_t));
+	secret_key = OQS_MEM_malloc(kem->length_secret_key + 2 * sizeof(magic_t));
+	ciphertext = OQS_MEM_malloc(kem->length_ciphertext + 2 * sizeof(magic_t));
+	shared_secret_e = OQS_MEM_malloc(kem->length_shared_secret + 2 * sizeof(magic_t));
+	shared_secret_d = OQS_MEM_malloc(kem->length_shared_secret + 2 * sizeof(magic_t));
+	seed = malloc(kem->length_keypair_seed + 2 * sizeof(magic_t));
 
 	if ((public_key == NULL) || (secret_key == NULL) || (ciphertext == NULL) || (shared_secret_e == NULL) || (shared_secret_d == NULL)) {
-		fprintf(stderr, "ERROR: malloc failed\n");
+		fprintf(stderr, "ERROR: OQS_MEM_malloc failed\n");
 		goto err;
 	}
 
@@ -78,12 +158,14 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 	memcpy(ciphertext, magic.val, sizeof(magic_t));
 	memcpy(shared_secret_e, magic.val, sizeof(magic_t));
 	memcpy(shared_secret_d, magic.val, sizeof(magic_t));
+	memcpy(seed, magic.val, sizeof(magic_t));
 
 	public_key += sizeof(magic_t);
 	secret_key += sizeof(magic_t);
 	ciphertext += sizeof(magic_t);
 	shared_secret_e += sizeof(magic_t);
 	shared_secret_d += sizeof(magic_t);
+	seed += sizeof(magic_t);
 
 	// and after
 	memcpy(public_key + kem->length_public_key, magic.val, sizeof(magic_t));
@@ -91,12 +173,38 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 	memcpy(ciphertext + kem->length_ciphertext, magic.val, sizeof(magic_t));
 	memcpy(shared_secret_e + kem->length_shared_secret, magic.val, sizeof(magic_t));
 	memcpy(shared_secret_d + kem->length_shared_secret, magic.val, sizeof(magic_t));
+	memcpy(seed + kem->length_keypair_seed, magic.val, sizeof(magic_t));
 
-	rc = OQS_KEM_keypair(kem, public_key, secret_key);
-	OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
-	if (rc != OQS_SUCCESS) {
-		fprintf(stderr, "ERROR: OQS_KEM_keypair failed\n");
-		goto err;
+
+	if (derand) {
+		// On some systems, getentropy fails if given a zero-length array
+		if (kem->length_keypair_seed > 0) {
+			OQS_randombytes(seed, kem->length_keypair_seed);
+		}
+		rc = OQS_KEM_keypair_derand(kem, public_key, secret_key, seed);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (kem->length_keypair_seed == 0) {
+			// If length_keypair_seed is set to 0 for this KEM scheme, a failure is expected
+			if (rc != OQS_ERROR) {
+				fprintf(stderr, "ERROR: OQS_KEM_keypair_derand succeeded but expected a failure\n");
+				goto err;
+			}
+			printf("OQS_KEM_keypair_derand failed, as expected\n");
+			ret = OQS_SUCCESS;
+			goto cleanup;
+		} else {
+			if (rc != OQS_SUCCESS) {
+				fprintf(stderr, "ERROR: OQS_KEM_keypair_derand failed\n");
+				goto err;
+			}
+		}
+	} else {
+		rc = OQS_KEM_keypair(kem, public_key, secret_key);
+		OQS_TEST_CT_DECLASSIFY(&rc, sizeof rc);
+		if (rc != OQS_SUCCESS) {
+			fprintf(stderr, "ERROR: OQS_KEM_keypair failed\n");
+			goto err;
+		}
 	}
 
 	OQS_TEST_CT_DECLASSIFY(public_key, kem->length_public_key);
@@ -127,6 +235,13 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 		printf("shared secrets are equal\n");
 	}
 
+#ifdef OQS_ENABLE_KEM_ML_KEM
+	/* check mlkem rejection testcases. returns true for all other kem algos */
+	if (false == mlkem_rej_testcase(kem, ciphertext, secret_key)) {
+		goto err;
+	}
+#endif
+
 	// test invalid encapsulation (call should either fail or result in invalid shared secret)
 	OQS_randombytes(ciphertext, kem->length_ciphertext);
 	OQS_TEST_CT_DECLASSIFY(ciphertext, kem->length_ciphertext);
@@ -144,11 +259,13 @@ static OQS_STATUS kem_test_correctness(const char *method_name) {
 	rv |= memcmp(ciphertext + kem->length_ciphertext, magic.val, sizeof(magic_t));
 	rv |= memcmp(shared_secret_e + kem->length_shared_secret, magic.val, sizeof(magic_t));
 	rv |= memcmp(shared_secret_d + kem->length_shared_secret, magic.val, sizeof(magic_t));
+	rv |= memcmp(seed + kem->length_keypair_seed, magic.val, sizeof(magic_t));
 	rv |= memcmp(public_key - sizeof(magic_t), magic.val, sizeof(magic_t));
 	rv |= memcmp(secret_key - sizeof(magic_t), magic.val, sizeof(magic_t));
 	rv |= memcmp(ciphertext - sizeof(magic_t), magic.val, sizeof(magic_t));
 	rv |= memcmp(shared_secret_e - sizeof(magic_t), magic.val, sizeof(magic_t));
 	rv |= memcmp(shared_secret_d - sizeof(magic_t), magic.val, sizeof(magic_t));
+	rv |= memcmp(seed - sizeof(magic_t), magic.val, sizeof(magic_t));
 	if (rv != 0) {
 		fprintf(stderr, "ERROR: Magic numbers do not match\n");
 		goto err;
@@ -177,6 +294,9 @@ cleanup:
 	if (ciphertext) {
 		OQS_MEM_insecure_free(ciphertext - sizeof(magic_t));
 	}
+	if (seed) {
+		OQS_MEM_secure_free(seed - sizeof(magic_t), kem->length_keypair_seed + 2 * sizeof(magic_t));
+	}
 	OQS_KEM_free(kem);
 
 	return ret;
@@ -197,7 +317,7 @@ static void TEST_KEM_randombytes(uint8_t *random_array, size_t bytes_to_read) {
 }
 #endif
 
-#if OQS_USE_PTHREADS_IN_TESTS
+#if OQS_USE_PTHREADS
 struct thread_data {
 	char *alg_name;
 	OQS_STATUS rc;
@@ -205,7 +325,12 @@ struct thread_data {
 
 void *test_wrapper(void *arg) {
 	struct thread_data *td = arg;
-	td->rc = kem_test_correctness(td->alg_name);
+	td->rc = kem_test_correctness(td->alg_name, false);
+	if (td->rc == OQS_SUCCESS) {
+		// test derandomized operations
+		td->rc = kem_test_correctness(td->alg_name, true);
+	}
+	OQS_thread_stop();
 	return NULL;
 }
 #endif
@@ -245,7 +370,7 @@ int main(int argc, char **argv) {
 #endif
 
 	OQS_STATUS rc;
-#if OQS_USE_PTHREADS_IN_TESTS
+#if OQS_USE_PTHREADS
 #define MAX_LEN_KEM_NAME_ 64
 	// don't run Classic McEliece in threads because of large stack usage
 	char no_thread_kem_patterns[][MAX_LEN_KEM_NAME_]  = {"Classic-McEliece", "HQC-256-"};
@@ -269,10 +394,18 @@ int main(int argc, char **argv) {
 		pthread_join(thread, NULL);
 		rc = td.rc;
 	} else {
-		rc = kem_test_correctness(alg_name);
+		rc = kem_test_correctness(alg_name, false);
+		if (rc == OQS_SUCCESS) {
+			// test with derandomized keygen
+			rc = kem_test_correctness(alg_name, true);
+		}
 	}
 #else
-	rc = kem_test_correctness(alg_name);
+	rc = kem_test_correctness(alg_name, false);
+	if (rc == OQS_SUCCESS) {
+		// test with derandomized keygen
+		rc = kem_test_correctness(alg_name, true);
+	}
 #endif
 	if (rc != OQS_SUCCESS) {
 		OQS_destroy();
